import { system, world,MolangVariableMap } from "@minecraft/server";
import {Attack} from "./items/spears.js"

function getScore(player, objective) {
  try {
    const s = world.scoreboard.getObjective(objective);
    if (!s) world.scoreboard.addObjective(objective, objective);
    return s.getScore(player.scoreboardIdentity)!=0?s.getScore(player.scoreboardIdentity):1;
  } catch {
    return 1;
  }
}

const artifactsList = [
  "poisonous_mucus",
  "corrupted_star",
  "blaze_heart",
  "blue_eye",
  "crystal_of_youth",
  "crystal_of_bravery",
  "crystal_of_rage",
  "crystal_of_fear",
  "dragon_heart",
  "battle_cry",
  'ancient_shield',
  'double_quiver',
  'bloody_totem',
  'explosive_blaze_rod',
  'soul_bottle',
  'blood_vial',
  'sculk_quiver',
  'magic_axe',
  'enchanted_socks',
  'magnetic_firework',
  'magnet',
  'ring_of_atlantis',
  'golden_ring',
  'heavy_quiver',
  'magma_socks',
  'noinsomnia',
  'heart_bottle',
  'gheart_bottle',
  'undying_flower',
  'bloody_ruby',
  'long_spear',
  "chaining_scroll",
  "charged_hit_scroll",
  "crushing_scroll",
  "uppercut_scroll",
  'sharper_spear',
  'speed_spear',
  'shield_spear',
  'wither_spear',
  'cloud_socks',
  'heavy_socks',
  'molten_ingot',
  'bloody_bramble',
  'sculk_portal',
  'supersonic_arrow',
  'void_socks',
  'chorus_socks',
  'purpur_socks',
  'void_cloak',
  'xp_orb'

]
system.runInterval(() => {

  for (const player of world.getPlayers()){
    
    if (player.hasTag("fly")) {
      player.applyKnockback(player.getViewDirection().x, player.getViewDirection().z, Math.sqrt(Math.abs(player.getViewDirection().x)*1.1 + Math.abs(player.getViewDirection().z)*1.1 )*0.2, 0.2)
    }
    if(player.hasTag('attack'))
    {
      player.removeTag('attack')
      Attack(player)
    }
    if(player.isSleeping && player.hasTag('blood_moon'))
    {
      player.runCommandAsync('tp @s ~~1~')
      player.onScreenDisplay.setActionBar("§cToo noisy to sleep");
    }

    const score = getScore(player,'Hit_cooldown')
    if(score>2)
    {
      if(player.hasTag('sword_spear_hand'))
      {
        var bar="|"
        var text = bar.repeat(Math.round((score-3)/2)) + "§8" +bar.repeat(10-Math.round((score-3)/2))
        player.onScreenDisplay.setActionBar(`${text}`);
      }
    }
    else if(score ==2)
      player.onScreenDisplay.setActionBar(" ");
    const fire = getScore(player,'Fire_breath')
    if(fire>0)
    {
      const options = { maxDistance: 7 ,excludeTypes: ["minecraft:item"],excludeFamilies:["projectile"],includePassableBlocks:false}
      var entities = world.getDimension(player.dimension.id).getEntitiesFromRay({x:player.getHeadLocation().x,y:player.getHeadLocation().y,z:player.getHeadLocation().z} /*Position de départ*/ , player.getViewDirection() /*Direction sale pute*/, options);
      for(const entity of entities)
      {
        if(entity.entity != player)
        {
            entity.entity.applyDamage(2,{cause: "fire",damagingEntity: player})
            entity.entity.setOnFire(4,true)
        }
      }
      const vmap = new MolangVariableMap();
      vmap.setVector3('variable.direction', player.getViewDirection());
      player.dimension.spawnParticle('edx:fire_breath',{x:player.getHeadLocation().x+player.getViewDirection().x/4,y:player.getHeadLocation().y+player.getViewDirection().y/4,z:player.getHeadLocation().z+player.getViewDirection().z/4},vmap);
      player.runCommand(`scoreboard players remove @s Fire_breath 1`)
    }
    const bow = getScore(player,'Bow')
    {
      if(bow>30)
      {
        const equipment = player.getComponent('equippable');
        const selectedItem = equipment.getEquipment('Mainhand');
        if(selectedItem.typeId == "edx:blaze_blade")
        {
          player.dimension.spawnParticle('edx:blaze_blade',{x:player.getHeadLocation().x,y:player.getHeadLocation().y,z:player.getHeadLocation().z});
        }
      }
    }
  }

 }
)
system.runInterval(() => {
  const dimensions = ['overworld', 'nether', 'the_end'].map(dim => world.getDimension(dim))
  for (const dim of dimensions){
    const potions = dim.getEntities({type: 'edx:area_incendiary_potion'});
    for (const potion of potions){
        let entities = dim.getEntities({location:{x:potion.location.x-3,y:potion.location.y,z:potion.location.z-3},volume:{x:6,y:1,z:6}});
        for(const entity of entities)
        {
          entity.setOnFire(10);
        }
    }
  }
},5)

system.runInterval(() => {
  const blood_moon = (((world.getAbsoluteTime() - world.getTimeOfDay())/24000)+2) % 8 == 0;
  const night_time = world.getTimeOfDay() > 12000 && world.getTimeOfDay() < 24000;
  if (blood_moon && night_time) {
    for (const player of world.getPlayers()){
      
      player.runCommand(`tag @s add blood_moon`);
    }
  }
  if (!blood_moon || !night_time) {
    for (const player of world.getPlayers()){
      player.runCommand(`tag @s remove blood_moon`);
    }
  }
  const entities = world.getDimension("minecraft:overworld").getEntities({excludeFamilies: ["projectile", "inanimate"] });
  for (const entity of entities){
    if (entity.hasTag("bleeding1")) {
      entity.removeTag("bleeding1");
      entity.runCommand(`damage @s 1 none`)
      entity.runCommand(`particle edx:blood ~~~`);
    }
    if (entity.hasTag("bleeding2")) {
      entity.removeTag("bleeding2");
      entity.runCommand(`tag @s add bleeding1`);
      entity.runCommand(`damage @s 1 none`)
      entity.runCommand(`particle edx:blood ~~~`);
    }
    if (entity.hasTag("bleeding3")) {
      entity.removeTag("bleeding3");
      entity.runCommand(`tag @s add bleeding2`);
      entity.runCommand(`damage @s 1 none`)
      entity.runCommand(`particle edx:blood ~~~`);
    }
    if (entity.hasTag("bleeding4")) {
      entity.removeTag("bleeding4");
      entity.runCommand(`tag @s add bleeding3`);
      entity.runCommand(`damage @s 2 none`)
      entity.runCommand(`particle edx:blood ~~~`);
    }
    if (entity.hasTag("bleeding")) {
      entity.removeTag("bleeding");
      entity.runCommand(`tag @s add bleeding4`);
      entity.runCommand(`damage @s 2 none`)
      entity.runCommand(`particle edx:blood ~~~`);
    }
  }


  for (const player of world.getPlayers()){
  if (player.hasTag("undying_flower") && player.hasTag("day")) {
    let health = player.getComponent("health")
    health.setCurrentValue(health.currentValue+0.33)
  }
  if (player.hasTag("fur_chestplate") && player.hasTag("frozen")) {
    let health = player.getComponent("health")
    health.setCurrentValue(health.currentValue+0.20)
  }
}
  },20)



system.runInterval(() => { 
  
  const players = world.getPlayers()
  for (let i = 0; i < players.length; i++) {
    let player = players[i]
    let inv = player.getComponent("inventory").container
    var item1= []
    var item2= []
    var item3= []
    if(inv.getItem(9)?.getTags()!=undefined )
      item1=inv.getItem(9)?.getTags()
    if(inv.getItem(18)?.getTags()!=undefined )
      item2=inv.getItem(18)?.getTags()
    if(inv.getItem(27)?.getTags()!=undefined )
      item3=inv.getItem(27)?.getTags()
    var goldenSlots = item1.concat(item2).concat(item3)
    if(goldenSlots.length>0)
      for(let j = 0; j < artifactsList.length; j++)
        {
          let hasArtifact = goldenSlots.includes(artifactsList[j])
          let hasTag = player.hasTag(artifactsList[j])
          if(hasTag && !hasArtifact)
            player.removeTag(artifactsList[j])
          else if(!hasTag && hasArtifact)
            player.addTag(artifactsList[j])
        }
      else
      for(let j = 0; j < artifactsList.length; j++)
        {
          player.removeTag(artifactsList[j])
        }
   
  }
  },40
)

function measureTime() {
  const startTime = Date.now()
  
  const executionTime = Date.now() - startTime
  console.warn(`Execution time: ${executionTime} milliseconds`);
}
